"""
This module contains constants that enables easy access to the ports on the SPIKE Prime hub. Use the constants in all functions that takes a `port` parameter.

To use the Port module add the following import statement to your project:

>>> from hub import port

All functions in the module should be called inside the port module as a prefix like so:

>>> port.A

"""

A = 0
"""The Port that is labelled 'A' on the Hub."""
B = 1
"""The Port that is labelled 'B' on the Hub."""
C = 2
"""The Port that is labelled 'C' on the Hub."""
D = 3
"""The Port that is labelled 'D' on the Hub."""
E = 4
"""The Port that is labelled 'E' on the Hub."""
F = 5
"""The Port that is labelled 'F' on the Hub."""
