"""
The `orientation` module contains all the orientation constants to use with the `light_matrix` module.

To use the `orientation` module add the following import statement to your project:

>>> import orientation
"""

UP = 0
RIGHT = 1
DOWN = 2
LEFT = 3
